/**************************************************************************************************

BaseConstraint.h -- (C) Niklas Een, Niklas Srensson, 2004

Defines the base constraint supported by UmSat.

**************************************************************************************************/

#ifndef BaseConstraint_h
#define BaseConstraint_h

#include "SolverTypes.h"

//=================================================================================================
// Constraint abstraction:


class Solver;

struct BaseConstraint {
  virtual void remove    (Solver& S, bool just_dealloc = false) = 0;
  virtual bool propagate (Solver& S, Lit p, bool& keep_watch) = 0;    // ('keep_watch' is set to FALSE beftore call to this method)
  virtual bool simplify  (Solver& S) { return false; };
  virtual void undo      (Solver& S, Lit p) { };
  virtual void calcReason(Solver& S, Lit p, vec<Lit>& out_reason) = 0;
  virtual const char * name(void) { return "N/A"; };
  
  virtual ~BaseConstraint(void) { };  // (not used, just to keep the compiler happy)
};

//=================================================================================================
// Helpers:

void removeWatch(vec<BaseConstraint*>& ws, BaseConstraint* elem);
void removeIfWatch(vec<BaseConstraint*>& ws, BaseConstraint* elem);

#endif
