/**************************************************************************************************

Clause.h -- (C) Niklas Een, Niklas Srensson, 2004

Defines the constraints supported by MiniSat (clauses being the core constraint). 

**************************************************************************************************/

#ifndef Clause_h
#define Clause_h

#include "BaseConstraint.h"
#include "SolverTypes.h"

//=================================================================================================
// Clauses:

class Clause : public BaseConstraint {
    unsigned    _size;
    int         _idx;

public:
    int  size    (void)         const { return _size; }
    void setSize (const vec<Lit>& ps) { _size = ps.size(); }
    int  getIdx  (void)         const { return _idx; }
    void setIdx  (int i)              { _idx = i; }
};

//=================================================================================================
// LClauses:

class LClause : public Clause {
    float       m_activity;
    Lit         data[0];

public:
    Lit  operator [] (int index) const { return data[index]; }
    
    // Constructor -- creates a new clause and add it to watcher lists. 
    friend bool LClause_new(Solver& S, const vec<Lit>& ps, LClause*& out_clause);

    // Learnt clauses only:
    bool    locked  (const Solver& S) const;
    float&  activity(void) const { return (float&)m_activity; }
    
    void remove    (Solver& S, bool just_dealloc = false);
    bool propagate (Solver& S, Lit p, bool& keep_watch);
    bool simplify  (Solver& S);
    void calcReason(Solver& S, Lit p, vec<Lit>& out_reason);
    const char *name(void) { return "LClause"; }
};

//=================================================================================================
// SClauses:


class SClause : public Clause {
  Lit         data[0];
  
 public:
  Lit  operator [] (int index) const { return data[index]; }
  
  // Constructor -- creates a new clause and add it to watcher lists. 
  friend bool SClause_new(Solver& S, const vec<Lit>& ps, SClause*& out_clause);
  
  void remove    (Solver& S, bool just_dealloc = false);
  bool propagate (Solver& S, Lit p, bool& keep_watch);
  bool simplify  (Solver& S);
  void calcReason(Solver& S, Lit p, vec<Lit>& out_reason);
  const char *name(void) { return "SClause"; }
};

#endif
