#include "SolverTypes.h"
#include "ConstraintsDB.h"
#include "Clause.h"
#include "Queue.h"
#include "VarOrder.h"
#include "Sort.h"

/*_________________________________________________________________________________________________
|                                                                                                  
|  reduceDB : ()  ->  [void]                                                                       
|                                                                                                  
|  Description:                                                                                    
|    Remove half of the learnt clauses, minus the clauses locked by the current assignment. Locked 
|    clauses are clauses that are reason to a some assignment.                                     
|________________________________________________________________________________________________@*/
struct reduceDB_lt { bool operator () (LClause* x, LClause* y) { return x->activity() < y->activity(); } };
void ConstraintsDB<LClause*>::reduceDB(Solver& S)
{
    int     i, j;
    double  extra_lim = cla_inc / size();    // Remove any clause below this activity

    sort(constrs, reduceDB_lt());
    for (i = j = 0; i < size() / 2; i++){
        if (!constrs[i]->locked(S))
            remove(S, i);
        else
            constrs[j++] = constrs[i];
    }
    for (; i < size(); i++){
        if (!constrs[i]->locked(S) && constrs[i]->activity() < extra_lim)
            remove(S, i);
        else
            constrs[j++] = constrs[i];
    }
    constrs.shrink(i - j);
}

/*_________________________________________________________________________________________________
|                                                                                                  
|  simplifyDB : [void]  ->  [void]                                                                 
|                                                                                                  
|  Description:                                                                                    
|    Simplify all constraints according to the current top-level assigment (redundant constraints  
|    may be removed altogether).                                                                   
|________________________________________________________________________________________________@*/

void ConstraintsDB<LClause*>::simplifyDB(Solver& S)
{
  int j = 0;
  for (int i = 0; i < size(); i++){
    if (constrs[i]->simplify(S))
      remove(S,i);
    else
      constrs[j++] = constrs[i];
  }
  constrs.shrink(constrs.size()-j);
}

//=================================================================================================

// Divide all variable activities by 1e100.
//
void ConstraintsDB<LClause*>::varRescaleActivity(void)
{
    for (int i = 0; i < activity.size(); i++)
        activity[i] *= 1e-100;
    var_inc *= 1e-100;
}


//=================================================================================================

// Divide all constraint activities by 1e20.
//
void ConstraintsDB<LClause*>::claRescaleActivity(void)
{
    for (int i = 0; i < size(); i++)
        constrs[i]->activity() *= 1e-20;
    cla_inc *= 1e-20;
}
