/**************************************************************************************************

Imply.C -- (C) Daniel Andrn, 2004

Defines the constraints Imply supported by UmSat.

**************************************************************************************************/

#include "Imply.h"
#include "Solver.h"
#include "Sort.h"

/***
//=================================================================================================
// Debug:


// For derivation output (verbosity level 2)
#define L_IND    "%-*d"
#define L_ind    S.decisionLevel()*3+3,S.decisionLevel()
#define L_LIT    "%sx%d"
#define L_lit(p) sign(p)?"~":"", var(p)
***/

//=================================================================================================
// Imply constraint:


// Returns FALSE if top-level conflict detected (must be handled); TRUE otherwise.
// 'out_clause' may be set to NULL if clause is already satisfied by the top-level assignment.
//
bool Imply_new(Solver& S, const Lit c, const vec<Lit>& ps, Imply*& out_clause)
{
    if (&out_clause != NULL) out_clause = NULL;

    assert(S.decisionLevel() == 0);

    if (ps.size() == 0)
        return false;
    //else if (ps.size() == 0)
    //  return S.enqueue(~c);
    else{
      // Allocate clause:
      assert(sizeof(Lit)   == sizeof(unsigned));
      assert(sizeof(float) == sizeof(unsigned));
      void* mem = xmalloc<char>(sizeof(Imply) + sizeof(unsigned)*ps.size());
      Imply*  x   = new (mem) Imply;

      x->size = ps.size();
      for (int i = 0; i < ps.size(); i++) x->lits[i] = ps[i];
      x->c = c;

      // Store And:
      S.watches[index(~x->c)].push(x);      
      for(int i = 0; i < ps.size(); i++) {
	S.watches[index(x->lits[i])].push(x);
      }

      if (&out_clause != NULL) out_clause = x;

      return true;
    }
}


void Imply::remove(Solver& S, bool just_dealloc)
{
  if (!just_dealloc){
    removeWatch(S.watches[index(~c)],this);
    for(int i = 0; i < this->size; ++i) {
      removeWatch(S.watches[index(lits[i])], this);
    }
  }
  xfree(this);
}


bool Imply::simplify(Solver& S)
{
  assert(S.decisionLevel() == 0);
  return false;
}


inline const char *myv(Solver& S, Lit l) { 
  return (S.value(l) == l_Undef ? "Undef" : (S.value(l) == l_True ? "True" : "False"));
}

// 'p' is the literal that became TRUE
inline bool Imply::propagate(Solver& S, Lit p, bool& keep_watch)
{
  // To collect some statistics!
  ++S.stats.prop[0];
  ++S.stats.propi;

  keep_watch = true;

  if(p == ~c) {
    for(int i = 0; i < size; ++i)
      if(!S.enqueue(~lits[i],this)) return false;
  } else {
    return S.enqueue(c,this);
  }

  return true;
}

void Imply::undo(Solver& S, Lit p) {}

// Can assume 'out_reason' to be empty.
// Calculate reason for 'p'. If 'p == lit_Undef', calculate reason for conflict.
//
void Imply::calcReason(Solver& S, Lit p, vec<Lit>& out_reason)
{
  if(p != c) out_reason.push(~c);
  if( p == c || p == lit_Undef ) 
    for(int i = 0; i < size; ++i)
      if(S.value(lits[i]) == l_True) {
	out_reason.push(lits[i]);
	break;
      }
  return;
}
