/**************************************************************************************************

Lex.C -- (C) Daniel Andrn, 2004

Defines the constraint Lex supported by UmSat.

**************************************************************************************************/

#include "Lex.h"
#include "Solver.h"

//=================================================================================================
// Lex constraint:

inline void Lex::add_watch(Solver& S, int i) {
  // fprintf(stderr,"Lex::add_watch(S, %d)\n", i);
  for(int j = max_watches; j < i; ++j) {
    S.watches[index(~lits[2*j])].push(this);
    // S.watches[index(lits[2*j])].push(this);
    // S.watches[index(~lits[2*j+1])].push(this);
    S.watches[index(lits[2*j+1])].push(this);
  }
  /***
  for(int j = i+1; j < max_watches; ++j) {
    removeIfWatch(S.watches[index(~lits[2*j])], this);
    removeIfWatch(S.watches[index(lits[2*j+1])], this);
  }

  max_watches = i ; //> max_watches ? i : max_watches;
  ***/
  max_watches = i > max_watches ? i : max_watches;
}

Lex *Lex::Lex_factory(Solver &S, const vec<Lit>& ps, const vec<Lit>& qs) {
  assert(S.decisionLevel() == 0);
  
  assert(ps.size() == qs.size());
  // Allocate constraint:
  assert(sizeof(Lit)   == sizeof(unsigned));
  void* mem = xmalloc<char>(sizeof(Lex) + sizeof(unsigned)*(ps.size() + qs.size()));
  Lex*  x   = new (mem) Lex;
  
  x->size = ps.size();
  for (int i = 0; i < ps.size(); ++i) {x->lits[2*i] = ps[i]; x->lits[2*i+1] = qs[i];}
  
  x->max_watches = 0;
  x->add_watch(S, 2);
  
  return x;
}

// Returns FALSE if top-level conflict detected (must be handled); TRUE otherwise.
// 'out_clause' may be set to NULL if clause is already satisfied by the top-level assignment.
//
bool Lex_new(Solver& S, const vec<Lit>& ps, const vec<Lit>& qs, Lex*& out_clause)
{
    if (&out_clause != NULL) out_clause = NULL;

    assert(S.decisionLevel() == 0);

    if (ps.size() == 0)
        return true;
    else{
      assert(ps.size() == qs.size());
      // Allocate constraint:
      assert(sizeof(Lit)   == sizeof(unsigned));
      void* mem = xmalloc<char>(sizeof(Lex) + sizeof(unsigned)*(ps.size() + qs.size()));
      Lex*  x   = new (mem) Lex;

      x->size = ps.size();
      for (int i = 0; i < ps.size(); ++i) {x->lits[2*i] = ps[i]; x->lits[2*i+1] = qs[i];}

      x->max_watches = 0;
      x->add_watch(S, 2);

      if (&out_clause != NULL) out_clause = x;

      return true;
    }
}


void Lex::remove(Solver& S, bool just_dealloc)
{
  if (!just_dealloc){
    for(int i = 0; i < 2*size; ++i) {
      removeIfWatch(S.watches[index(lits[i])], this);
      removeIfWatch(S.watches[index(~lits[i])], this);
    }
  }
  xfree(this);
}

bool Lex::simplify(Solver& S)
{
  assert(S.decisionLevel() == 0);
  return false;
}

inline bool Lex::check(Solver& S, int i)
{
  return false;
  int j;
  bool ret = false;
  for(j=i; j < size; ++j) {
    lbool x1 = S.value(lits[2*j]);
    lbool x2 = S.value(lits[2*j+1]);
    if(x1 != l_False && x2 != l_True) break;
    if(x1 == l_False && x2 == l_True ){
      // add_watch(S, j); 
      ret = true; 
      break;
    }
  }

  // add_watch(S, j);
  return ret;
}

// 'p' is the literal that became TRUE
inline bool Lex::propagate(Solver& S, Lit p, bool& keep_watch)
{
  // To collect some statistics!
  ++S.stats.prop[0];
  ++S.stats.propj;

  keep_watch = true;
  
  for(int i=0; i<size; ++i) {
    lbool x1 = S.value(lits[2*i]);
    lbool x2 = S.value(lits[2*i+1]);
    
    if(x1 == l_True) {
      if(x2 == l_False) { // 10
	add_watch(S,i);
	return true;
      } else if(x2 == l_Undef) { // 1*
	if(check(S,i)) S.enqueue(~lits[2*i+1], this);
	// add_watch(S,i);
	return true;
      }
    } else if(x1 == l_False) {
      if(x2 == l_True) { // 01
	add_watch(S,i);
	return false;
      } else if(x2 == l_Undef) { // 0*
	S.enqueue(~lits[2*i+1],this);
      }
    } else if(x1 == l_Undef) {
      if(x2 == l_True) { // *1
	S.enqueue(lits[2*i],this);
      } else { // *0, **
	if(check(S,i)) {S.enqueue(lits[2*i], this); S.enqueue(~lits[2*i+1], this);}
	// add_watch(S,i);
	return true;
      }
    }
  }
  add_watch(S,size);
  return true;
}

void Lex::undo(Solver& S, Lit p) {}

// Can assume 'out_reason' to be empty.
// Calculate reason for 'p'. If 'p == lit_Undef', calculate reason for conflict.
//
void Lex::calcReason(Solver& S, Lit p, vec<Lit>& out_reason)
{
  for(int i=0, j=0; i==j && i < size; ++i) {
    if(S.value(lits[2*i])==l_False) {out_reason.push(~lits[2*i]); ++j;}
    if(S.value(lits[2*i+1])==l_True) {out_reason.push(lits[2*i+1]); ++j;}
    if(lits[2*i]==p || lits[2*i+1]==p) { ++j; }
  }

  return;
}
