/**************************************************************************************************

Lex.h -- (C) Daniel Andrn, 2004

Defines the constraint Lex supported by UmSat.

**************************************************************************************************/

#ifndef Lex_h
#define Lex_h

#include "BaseConstraint.h"
#include "SolverTypes.h"

//=================================================================================================
// Lex:


class Lex : public BaseConstraint {
    int     size;
    int     max_watches;
    Lit     lits[0];

    void add_watch(Solver& S, int i);
    bool check(Solver& S, int i);

    Lex(void) {};

public:
    // Constructor -- creates a new Lex-constraint and add it to watcher lists.
    friend bool Lex_new(Solver& S, const vec<Lit>& ps, const vec<Lit>& qs, Lex*& out);

    Lex *Lex_factory(Solver &S, const vec<Lit>& ps, const vec<Lit>& qs);

    // Constraint interface:
    void remove    (Solver& S, bool just_dealloc = false);
    bool propagate (Solver& S, Lit p, bool& keep_watch);
    bool simplify  (Solver& S);
    void undo      (Solver& S, Lit p);
    void calcReason(Solver& S, Lit p, vec<Lit>& out_reason);
    const char *name(void) { return "Lex"; }
    
 private:
};


//=================================================================================================

#endif
