/**************************************************************************************************

SClause.h -- (C) Niklas Een, Niklas Srensson, 2004

Defines the constraints supported by MiniSat (clauses being the core constraint). 

**************************************************************************************************/

#ifndef SClause_h
#define SClause_h

#include "BaseConstraint.h"
#include "SolverTypes.h"

//=================================================================================================
// SClauses:


class SClause : public BaseConstraint {
    unsigned    m_size;
    int         idx;
    Lit         data[0];

public:
    int  size        (void)      const { return m_size; }
    bool learnt      (void)      const { return false; }
    Lit  operator [] (int index) const { return data[index]; }

    // Constructor -- creates a new clause and add it to watcher lists. 
    friend bool SClause_new(Solver& S, const vec<Lit>& ps, SClause*& out_clause);

    // Constraint interface:
    void remove    (Solver& S, bool just_dealloc = false);
    bool propagate (Solver& S, Lit p, bool& keep_watch);
    bool simplify  (Solver& S);
    void calcReason(Solver& S, Lit p, vec<Lit>& out_reason);
    const char *name(void) { return "SClause"; }
};

#endif
