/**************************************************************************************************

Xor.C -- (C) Daniel Andrn, 2004

Defines the constraint Xor supported by UmSat.

**************************************************************************************************/

#include "Xor.h"
#include "Solver.h"
#include "Sort.h"

#if 0
//=================================================================================================
// Debug:


// For derivation output (verbosity level 2)
#define L_IND    "%-*d"
#define L_ind    S.decisionLevel()*3+3,S.decisionLevel()
#define L_LIT    "%sx%d"
#define L_lit(p) sign(p)?"~":"", var(p)
#endif

//=================================================================================================
// Xor constraint:


// Returns FALSE if top-level conflict detected (must be handled); TRUE otherwise.
// 'out_clause' may be set to NULL if clause is already satisfied by the top-level assignment.
//
bool Xor_new(Solver& S, const vec<Lit>& ps, Xor*& out_clause)
{
    if (&out_clause != NULL) out_clause = NULL;

    assert(S.decisionLevel() == 0);

    if (ps.size() == 0)
        return true;
    else if (ps.size() == 1)
      return S.enqueue(~ps[0]);
    else{
      // Allocate clause:
      assert(sizeof(Lit)   == sizeof(unsigned));
      assert(sizeof(float) == sizeof(unsigned));
      void* mem = xmalloc<char>(sizeof(Xor) + sizeof(unsigned)*ps.size());
      Xor*  x   = new (mem) Xor;

      x->size = ps.size();
      for (int i = 0; i < ps.size(); i++) x->lits[i] = ps[i];
      x->par = 1;
      for (int i = 0; i < ps.size(); i++) x->par ^= index(ps[i]);

      x->nvars = ps.size();

      // Store Xor:
      for(int i = 0; i < ps.size(); i++) {
	S.watches[index(x->lits[i])].push(x);
	S.watches[index(~x->lits[i])].push(x);
      }

      if (&out_clause != NULL) out_clause = x;

      return true;
    }
}


void Xor::remove(Solver& S, bool just_dealloc)
{
  if (!just_dealloc){
    for(int i = 0; i < this->size; ++i) {
      removeWatch(S.watches[index(lits[i])], this);
      removeWatch(S.watches[index(~lits[i])], this);
    }
  }
  xfree(this);
}


bool Xor::simplify(Solver& S)
{
  assert(S.decisionLevel() == 0);
  return false;
}


// 'p' is the literal that became TRUE
inline bool Xor::propagate(Solver& S, Lit p, bool& keep_watch)
{
  // To collect some statistics!
  ++S.stats.prop[0];
  ++S.stats.propx;

  keep_watch = true;

  --nvars;
  par ^= index(p);
  S.undos[var(p)].push(this);

  if(nvars == 1) return S.enqueue(toLit(par),this);
  if(nvars == 0 && par != 0) return false;

  return true;
}

void Xor::undo(Solver& S, Lit p) {
  ++nvars;
  par ^= index(p);
}

// Can assume 'out_reason' to be empty.
// Calculate reason for 'p'. If 'p == lit_Undef', calculate reason for conflict.
//
void Xor::calcReason(Solver& S, Lit p, vec<Lit>& out_reason)
{

  for(int i = 0; i < size; ++i) {
    if (var(lits[i]) != var(p)) {
      if(S.value(lits[i]) == l_True) out_reason.push(lits[i]);
      else out_reason.push(~lits[i]);
      assert(S.value(lits[i]) != l_Undef);
    }
  }
}
