#ifndef readers_h
#define readers_h

#include "SolverTypes.h"
#include "Parser.h"

//=================================================================================================

class reader {
 public:
  virtual void read(Solver&, parser&) = 0;
};

//=================================================================================================

class readXor : public reader {
  void read(Solver& sol, parser& p)
    {
      vec<Lit> v;
      p.vector(v);
      sol.addXor(v);
    }
};

//=================================================================================================

class readAtMost: public reader {
  void read(Solver& sol, parser& p)
    {
      vec<Lit> v;
      int n;
      if(p.lh == '0') {
	n = 0;
	p.skip0();
      } else n = p.sint();
      p.skipMEWS();
      p.vector(v);
      sol.addAtMost(v,n);    
    }
};

//=================================================================================================

class readcnf: public reader {
 public:
  void read(Solver& sol, parser& p)
    {
      vec<Lit> v;
      p.vector(v);
      sol.addSClause(v);
    }
};

//=================================================================================================

class readImply : public reader {
  void read(Solver& sol, parser& p)
    {
      Lit l = p.lit();
      p.skipMEWS();
      vec<Lit> v;
      p.vector(v);
      sol.addImply(l,v);
    }
};

//=================================================================================================

class readjmf : public reader {
  void read(Solver& sol, parser& p)
    {
      vec<Lit> v;
      p.vector(v);
      p.skipOEWS();
      vec<Lit> w;
      p.vector(w);
      sol.addjmf(v,w);
    }
};

//=================================================================================================

class readLex : public reader {
  void read(Solver& sol, parser& p)
    {
      vec<Lit> v;
      p.vector(v);
      p.skipOEWS();
      vec<Lit> w;
      p.vector(w);
      sol.addLex(v,w);
    }
};

//=================================================================================================

#endif
