/**************************************************************************************************

VarOrder.C -- (C) Niklas Een, Niklas Srensson, 2004

ADT for maintaining the variable ordering. It will keep a list of all variables sorted on the
current activity.  

**************************************************************************************************/

#include "Solver.h"
#include "VarOrder.h"
#include "Sort.h"

struct VarOrder_lt {
    const vec<double>&  activity;
    bool operator () (Var x, Var y) { return activity[x] < activity[y]; }
    VarOrder_lt(const vec<double>&  act) : activity(act) { }
};

void VarOrder::updateAll(void)
{
    VarOrder_lt lt(activity);
    sort(order, lt);    // <= Should really use stable sort here.
    for (int i = 0; i < order.size(); i++)
        varpos[order[i]] = i;
    last_var = 0;
}

Var VarOrder::select(Solver &S, double random_var_freq)
{
    // Random decision:
    if (drand(random_seed) < random_var_freq){
        Var next = order[irand(random_seed, order.size())];
        assert(next >= 0 && next < assigns.size());
        if (toLbool(assigns[next]) == l_Undef)
            return next;
    }

    /***
    //int min = 2*S.nVars()+1;
    int max = -1;
    Var var = var_Undef;
    for(int i = 0; i < S.nVars(); ++i) {
      int j = S.lenWatches(toLit(i)) + S.lenWatches(~toLit(i));
      if(j > max && toLbool(assigns[i]) == l_Undef) {
	max = j;
	var = i;
      }
    }
    return var;
    ***/

    // Activity based decision:
    for (int i = last_var; i < order.size(); i++){
        if (toLbool(assigns[order[i]]) == l_Undef){
            last_var = i;
            return order[i]; }
    }

    return var_Undef;
}

vec<int> VarOrder::getOrder(Solver &S) {
  vec<int> list;

  for (int i = 0; i < order.size(); ++i)
    if (toLbool(assigns[order[i]]) == l_Undef)
      list.push(order[i]);
  
  return list;
}
